from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius import JavaException, JavaClass
from jnius.reflect import autoclass

class BadDeclarationTest(unittest.TestCase):

    def test_class_not_found(self):
        #self.assertRaises(JavaException, autoclass, 'org.unknow.class')
        #self.assertRaises(JavaException, autoclass, 'java/lang/String')
        pass

    def test_invalid_attribute(self):
        Stack = autoclass('java.util.Stack')
        self.assertRaises(AttributeError, getattr, Stack, 'helloworld')

    def test_invalid_static_call(self):
        Stack = autoclass('java.util.Stack')
        self.assertRaises(JavaException, Stack.push, 'hello')

    def test_with_too_much_arguments(self):
        Stack = autoclass('java.util.Stack')
        stack = Stack()
        self.assertRaises(JavaException, stack.push, 'hello', 'world', 123)

    def test_java_exception_handling(self):
        Stack = autoclass('java.util.Stack')
        stack = Stack()
        try:
            stack.pop()
            self.fail("Expected exception to be thrown")
        except JavaException as je:
            # print "Got JavaException: " + str(je)
            # print "Got Exception Class: " + je.classname
            # print "Got stacktrace: \n" + '\n'.join(je.stacktrace)
            self.assertEqual("java.util.EmptyStackException", je.classname)

    def test_java_exception_chaining(self):
        BasicsTest = autoclass('org.jnius.BasicsTest')
        basics = BasicsTest()
        try:
            basics.methodExceptionChained()
            self.fail("Expected exception to be thrown")
        except JavaException as je:
            # print "Got JavaException: " + str(je)
            # print "Got Exception Class: " + je.classname
            # print "Got Exception Message: " + je.innermessage
            # print "Got stacktrace: \n" + '\n'.join(je.stacktrace)
            self.assertEqual("java.lang.IllegalArgumentException", je.classname)
            self.assertEqual("helloworld2", je.innermessage)
            self.assertIn("Caused by:", je.stacktrace)
            self.assertEqual(11, len(je.stacktrace))



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import sys
import unittest
import jnius_config
from jnius import JavaMultipleMethod, JavaException
from jnius.reflect import autoclass


try:
    long
except NameError:
    # Python 3
    long = int


def py2_encode(uni):
    if sys.version_info < (3, 0):
        uni = uni.encode('utf-8')
    return uni


class VisibilityPublicProtectedTest(unittest.TestCase):

    def test_static_fields_public_protected(self):
        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=False)

        self.assertTrue(hasattr(Test, 'fieldStaticPublic'))
        self.assertTrue(hasattr(Test, 'fieldStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'fieldStaticProtected'))
        self.assertFalse(hasattr(Test, 'fieldStaticPrivate'))

        self.assertEqual(Test.fieldStaticPublic, py2_encode("StaticPublic"))
        self.assertEqual(Test.fieldStaticPackageProtected, py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.fieldStaticProtected, py2_encode("StaticProtected"))

    def test_child_static_fields_public_protected(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=False)

        self.assertTrue(hasattr(Test, 'fieldStaticPublic'))
        self.assertTrue(hasattr(Test, 'fieldStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'fieldStaticProtected'))
        self.assertFalse(hasattr(Test, 'fieldStaticPrivate'))

        self.assertEqual(Test.fieldStaticPublic, py2_encode("StaticPublic"))
        self.assertEqual(Test.fieldStaticPackageProtected, py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.fieldStaticProtected, py2_encode("StaticProtected"))

        self.assertTrue(hasattr(Test, 'fieldChildStaticPublic'))
        self.assertTrue(hasattr(Test, 'fieldChildStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'fieldChildStaticProtected'))
        self.assertFalse(hasattr(Test, 'fieldChildStaticPrivate'))

        self.assertEqual(Test.fieldChildStaticPublic, py2_encode("ChildStaticPublic"))
        self.assertEqual(Test.fieldChildStaticPackageProtected, py2_encode("ChildStaticPackageProtected"))
        self.assertEqual(Test.fieldChildStaticProtected, py2_encode("ChildStaticProtected"))

    def test_static_methods_public_protected(self):
        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=False)

        self.assertTrue(hasattr(Test, 'methodStaticPublic'))
        self.assertTrue(hasattr(Test, 'methodStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'methodStaticProtected'))
        self.assertFalse(hasattr(Test, 'methodStaticPrivate'))

        self.assertEqual(Test.methodStaticPublic(), py2_encode("StaticPublic"))
        self.assertEqual(Test.methodStaticPackageProtected(), py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.methodStaticProtected(), py2_encode("StaticProtected"))

    def test_child_static_methods_public_protected(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=False)

        self.assertTrue(hasattr(Test, 'methodStaticPublic'))
        self.assertTrue(hasattr(Test, 'methodStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'methodStaticProtected'))
        self.assertFalse(hasattr(Test, 'methodStaticPrivate'))

        self.assertEqual(Test.methodStaticPublic(), py2_encode("StaticPublic"))
        self.assertEqual(Test.methodStaticPackageProtected(), py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.methodStaticProtected(), py2_encode("StaticProtected"))

        self.assertTrue(hasattr(Test, 'methodChildStaticPublic'))
        self.assertTrue(hasattr(Test, 'methodChildStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'methodChildStaticProtected'))
        self.assertFalse(hasattr(Test, 'methodChildStaticPrivate'))

        self.assertEqual(Test.methodChildStaticPublic(), py2_encode("ChildStaticPublic"))
        self.assertEqual(Test.methodChildStaticPackageProtected(), py2_encode("ChildStaticPackageProtected"))
        self.assertEqual(Test.methodChildStaticProtected(), py2_encode("ChildStaticProtected"))

    def test_fields_public_protected(self):

        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=False)
        test = Test()

        self.assertTrue(hasattr(test, 'fieldPublic'))
        self.assertTrue(hasattr(test, 'fieldPackageProtected'))
        self.assertTrue(hasattr(test, 'fieldProtected'))
        self.assertFalse(hasattr(test, 'fieldPrivate'))

        self.assertEqual(test.fieldPublic, py2_encode("Public"))
        self.assertEqual(test.fieldPackageProtected, py2_encode("PackageProtected"))
        self.assertEqual(test.fieldProtected, py2_encode("Protected"))

    def test_child_fields_public_protected(self):

        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=False)
        test = Test()

        self.assertTrue(hasattr(test, 'fieldPublic'))
        self.assertTrue(hasattr(test, 'fieldPackageProtected'))
        self.assertTrue(hasattr(test, 'fieldProtected'))
        self.assertFalse(hasattr(test, 'fieldPrivate'))

        self.assertEqual(test.fieldPublic, py2_encode("Public"))
        self.assertEqual(test.fieldPackageProtected, py2_encode("PackageProtected"))
        self.assertEqual(test.fieldProtected, py2_encode("Protected"))

        self.assertTrue(hasattr(test, 'fieldChildPublic'))
        self.assertTrue(hasattr(test, 'fieldChildPackageProtected'))
        self.assertTrue(hasattr(test, 'fieldChildProtected'))
        self.assertFalse(hasattr(test, 'fieldChildPrivate'))

        self.assertEqual(test.fieldChildPublic, py2_encode("ChildPublic"))
        self.assertEqual(test.fieldChildPackageProtected, py2_encode("ChildPackageProtected"))
        self.assertEqual(test.fieldChildProtected, py2_encode("ChildProtected"))

    def test_methods_public_protected(self):

        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=False)
        test = Test()

        self.assertTrue(hasattr(test, 'methodPublic'))
        self.assertTrue(hasattr(test, 'methodPackageProtected'))
        self.assertTrue(hasattr(test, 'methodProtected'))
        self.assertFalse(hasattr(test, 'methodPrivate'))

        self.assertEqual(test.methodPublic(), py2_encode("Public"))
        self.assertEqual(test.methodPackageProtected(), py2_encode("PackageProtected"))
        self.assertEqual(test.methodProtected(), py2_encode("Protected"))

    def test_child_methods_public_protected(self):

        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=False)
        test = Test()

        self.assertTrue(hasattr(test, 'methodPublic'))
        self.assertTrue(hasattr(test, 'methodPackageProtected'))
        self.assertTrue(hasattr(test, 'methodProtected'))
        self.assertFalse(hasattr(test, 'methodPrivate'))

        self.assertEqual(test.methodPublic(), py2_encode("Public"))
        self.assertEqual(test.methodPackageProtected(), py2_encode("PackageProtected"))
        self.assertEqual(test.methodProtected(), py2_encode("Protected"))

        self.assertTrue(hasattr(test, 'methodChildPublic'))
        self.assertTrue(hasattr(test, 'methodChildPackageProtected'))
        self.assertTrue(hasattr(test, 'methodChildProtected'))
        self.assertFalse(hasattr(test, 'methodChildPrivate'))

        self.assertEqual(test.methodChildPublic(), py2_encode("ChildPublic"))
        self.assertEqual(test.methodChildPackageProtected(), py2_encode("ChildPackageProtected"))
        self.assertEqual(test.methodChildProtected(), py2_encode("ChildProtected"))

    def test_static_multi_methods(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=False)

        self.assertTrue(hasattr(Test, 'methodStaticMultiArgs'))
        self.assertTrue(isinstance(Test.methodStaticMultiArgs, JavaMultipleMethod))

        self.assertTrue(Test.methodStaticMultiArgs(True))
        self.assertTrue(Test.methodStaticMultiArgs(True, False))
        with self.assertRaises(JavaException):
            Test.methodStaticMultiArgs(True, False, True)

    def test_multi_methods(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=False)
        test = Test()

        self.assertTrue(hasattr(test, 'methodMultiArgs'))
        self.assertTrue(isinstance(Test.methodMultiArgs, JavaMultipleMethod))

        self.assertTrue(test.methodMultiArgs(True))
        self.assertTrue(test.methodMultiArgs(True, False))
        with self.assertRaises(JavaException):
           test.methodMultiArgs(True, False, True)



from __future__ import absolute_import
import unittest
from jnius import autoclass, protocol_map


class TestCollections(unittest.TestCase):

    def test_hashset(self):
        hset = autoclass('java.util.HashSet')()
        data = {1,2}
        # add is in both Python and Java
        for k in data:
            hset.add(k)
        # __len__
        print(dir(hset))
        self.assertEqual(len(data), len(hset))
        # __contains__
        for k in data:
            self.assertTrue(k in hset)
        self.assertFalse(0 in hset)
        # __iter__
        for k in hset:
            self.assertTrue(k in data)
        # __delitem__
        for k in data:
            del(hset[k])
            self.assertFalse(k in hset)
        
    def test_hashmap(self):
        hmap = autoclass('java.util.HashMap')()
        data = {1 : 'hello', 2 : 'world'}
        # __setitem__
        for k,v in data.items():
            hmap[k] = v
        # __len__
        self.assertEqual(len(data), len(hmap))
        # __contains__
        for k,v in data.items():
            self.assertTrue(k in hmap)
            self.assertEqual(data[k], hmap[k])
        # __iter__
        for k in hmap:
           self.assertTrue(k in data)
        # __contains__
        self.assertFalse(0 in hmap)
        # __delitem__
        for k in data:
            del(hmap[k])
            self.assertFalse(k in hmap)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import json
import pytest
import subprocess
import sys
import textwrap


class TestJVMOptions:
    @pytest.mark.skipif(
        sys.platform == 'android',
        reason='JNIus on Android does not take JVM options'
    )
    def test_jvm_options(self):
        options = ['-Dtest.var{}=value'.format(i) for i in range(40)]
        process = subprocess.Popen(['python', '-c', textwrap.dedent(
            '''\
            import jnius_config
            import json
            import sys
            jnius_config.set_options(*json.load(sys.stdin))
            import jnius
            ManagementFactory = jnius.autoclass("java.lang.management.ManagementFactory")
            json.dump(list(ManagementFactory.getRuntimeMXBean().getInputArguments()), sys.stdout)''')],
            bufsize=-1, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
        stdoutdata, _ = process.communicate(json.dumps(options).encode())
        assert process.wait() == 0
        actual_options = json.loads(stdoutdata.decode())
        assert list(sorted(options)) == list(sorted(actual_options))



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import sys
import unittest
import jnius_config
from jnius import JavaMultipleMethod, JavaMethod
from jnius.reflect import autoclass


try:
    long
except NameError:
    # Python 3
    long = int


def py2_encode(uni):
    if sys.version_info < (3, 0):
        uni = uni.encode('utf-8')
    return uni


class VisibilityAllTest(unittest.TestCase):

    def test_static_fields_all(self):
        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=True)

        self.assertTrue(hasattr(Test, 'fieldStaticPublic'))
        self.assertTrue(hasattr(Test, 'fieldStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'fieldStaticProtected'))
        self.assertTrue(hasattr(Test, 'fieldStaticPrivate'))

        self.assertEqual(Test.fieldStaticPublic, py2_encode("StaticPublic"))
        self.assertEqual(Test.fieldStaticPackageProtected, py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.fieldStaticProtected, py2_encode("StaticProtected"))
        self.assertEqual(Test.fieldStaticPrivate, py2_encode("StaticPrivate"))

    def test_child_static_fields_all(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=True)

        self.assertTrue(hasattr(Test, 'fieldStaticPublic'))
        self.assertTrue(hasattr(Test, 'fieldStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'fieldStaticProtected'))
        self.assertTrue(hasattr(Test, 'fieldStaticPrivate'))

        self.assertEqual(Test.fieldStaticPublic, py2_encode("StaticPublic"))
        self.assertEqual(Test.fieldStaticPackageProtected, py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.fieldStaticProtected, py2_encode("StaticProtected"))
        self.assertEqual(Test.fieldStaticPrivate, py2_encode("StaticPrivate"))

        self.assertTrue(hasattr(Test, 'fieldChildStaticPublic'))
        self.assertTrue(hasattr(Test, 'fieldChildStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'fieldChildStaticProtected'))
        self.assertTrue(hasattr(Test, 'fieldChildStaticPrivate'))

        self.assertEqual(Test.fieldChildStaticPublic, py2_encode("ChildStaticPublic"))
        self.assertEqual(Test.fieldChildStaticPackageProtected, py2_encode("ChildStaticPackageProtected"))
        self.assertEqual(Test.fieldChildStaticProtected, py2_encode("ChildStaticProtected"))
        self.assertEqual(Test.fieldChildStaticPrivate, py2_encode("ChildStaticPrivate"))

    def test_static_methods_all(self):
        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=True)

        self.assertTrue(hasattr(Test, 'methodStaticPublic'))
        self.assertTrue(hasattr(Test, 'methodStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'methodStaticProtected'))
        self.assertTrue(hasattr(Test, 'methodStaticPrivate'))

        self.assertEqual(Test.methodStaticPublic(), py2_encode("StaticPublic"))
        self.assertEqual(Test.methodStaticPackageProtected(), py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.methodStaticProtected(), py2_encode("StaticProtected"))
        self.assertEqual(Test.methodStaticPrivate(), py2_encode("StaticPrivate"))

    def test_child_static_methods_all(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=True)

        self.assertTrue(hasattr(Test, 'methodStaticPublic'))
        self.assertTrue(hasattr(Test, 'methodStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'methodStaticProtected'))
        self.assertTrue(hasattr(Test, 'methodStaticPrivate'))

        self.assertEqual(Test.methodStaticPublic(), py2_encode("StaticPublic"))
        self.assertEqual(Test.methodStaticPackageProtected(), py2_encode("StaticPackageProtected"))
        self.assertEqual(Test.methodStaticProtected(), py2_encode("StaticProtected"))
        self.assertEqual(Test.methodStaticPrivate(), py2_encode("StaticPrivate"))

        self.assertTrue(hasattr(Test, 'methodChildStaticPublic'))
        self.assertTrue(hasattr(Test, 'methodChildStaticPackageProtected'))
        self.assertTrue(hasattr(Test, 'methodChildStaticProtected'))
        self.assertTrue(hasattr(Test, 'methodChildStaticPrivate'))

        self.assertEqual(Test.methodChildStaticPublic(), py2_encode("ChildStaticPublic"))
        self.assertEqual(Test.methodChildStaticPackageProtected(), py2_encode("ChildStaticPackageProtected"))
        self.assertEqual(Test.methodChildStaticProtected(), py2_encode("ChildStaticProtected"))
        self.assertEqual(Test.methodChildStaticPrivate(), py2_encode("ChildStaticPrivate"))

    def test_fields_all(self):
        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=True)
        test = Test()

        self.assertTrue(hasattr(test, 'fieldPublic'))
        self.assertTrue(hasattr(test, 'fieldPackageProtected'))
        self.assertTrue(hasattr(test, 'fieldProtected'))
        self.assertTrue(hasattr(test, 'fieldPrivate'))

        self.assertEqual(test.fieldPublic, py2_encode("Public"))
        self.assertEqual(test.fieldPackageProtected, py2_encode("PackageProtected"))
        self.assertEqual(test.fieldProtected, py2_encode("Protected"))
        self.assertEqual(test.fieldPrivate, py2_encode("Private"))

    def test_child_fields_all(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=True)
        test = Test()

        self.assertTrue(hasattr(test, 'fieldPublic'))
        self.assertTrue(hasattr(test, 'fieldPackageProtected'))
        self.assertTrue(hasattr(test, 'fieldProtected'))
        self.assertTrue(hasattr(test, 'fieldPrivate'))

        self.assertEqual(test.fieldPublic, py2_encode("Public"))
        self.assertEqual(test.fieldPackageProtected, py2_encode("PackageProtected"))
        self.assertEqual(test.fieldProtected, py2_encode("Protected"))
        self.assertEqual(test.fieldPrivate, py2_encode("Private"))

        self.assertTrue(hasattr(test, 'fieldChildPublic'))
        self.assertTrue(hasattr(test, 'fieldChildPackageProtected'))
        self.assertTrue(hasattr(test, 'fieldChildProtected'))
        self.assertTrue(hasattr(test, 'fieldChildPrivate'))

        self.assertEqual(test.fieldChildPublic, py2_encode("ChildPublic"))
        self.assertEqual(test.fieldChildPackageProtected, py2_encode("ChildPackageProtected"))
        self.assertEqual(test.fieldChildProtected, py2_encode("ChildProtected"))
        self.assertEqual(test.fieldChildPrivate, py2_encode("ChildPrivate"))

    def test_methods_all(self):
        Test = autoclass('org.jnius.VisibilityTest', include_protected=True, include_private=True)
        test = Test()

        self.assertTrue(hasattr(test, 'methodPublic'))
        self.assertTrue(hasattr(test, 'methodPackageProtected'))
        self.assertTrue(hasattr(test, 'methodProtected'))
        self.assertTrue(hasattr(test, 'methodPrivate'))

        self.assertEqual(test.methodPublic(), py2_encode("Public"))
        self.assertEqual(test.methodPackageProtected(), py2_encode("PackageProtected"))
        self.assertEqual(test.methodProtected(), py2_encode("Protected"))
        self.assertEqual(test.methodPrivate(), py2_encode("Private"))

    def test_child_methods_all(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=True)
        test = Test()
        self.assertTrue(hasattr(test, 'methodPublic'))
        self.assertTrue(hasattr(test, 'methodPackageProtected'))
        self.assertTrue(hasattr(test, 'methodProtected'))
        self.assertTrue(hasattr(test, 'methodPrivate'))

        self.assertEqual(test.methodPublic(), py2_encode("Public"))
        self.assertEqual(test.methodPackageProtected(), py2_encode("PackageProtected"))
        self.assertEqual(test.methodProtected(), py2_encode("Protected"))
        self.assertEqual(test.methodPrivate(), py2_encode("Private"))

        self.assertTrue(hasattr(test, 'methodChildPublic'))
        self.assertTrue(hasattr(test, 'methodChildPackageProtected'))
        self.assertTrue(hasattr(test, 'methodChildProtected'))
        self.assertTrue(hasattr(test, 'methodChildPrivate'))

        self.assertEqual(test.methodChildPublic(), py2_encode("ChildPublic"))
        self.assertEqual(test.methodChildPackageProtected(), py2_encode("ChildPackageProtected"))
        self.assertEqual(test.methodChildProtected(), py2_encode("ChildProtected"))
        self.assertEqual(test.methodChildPrivate(), py2_encode("ChildPrivate"))

    def test_static_multi_methods(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=True)

        self.assertTrue(hasattr(Test, 'methodStaticMultiArgs'))
        self.assertTrue(isinstance(Test.methodStaticMultiArgs, JavaMultipleMethod))

        self.assertTrue(Test.methodStaticMultiArgs(True))
        self.assertTrue(Test.methodStaticMultiArgs(True, False))
        self.assertTrue(Test.methodStaticMultiArgs(True, False, True))

    def test_multi_methods(self):
        Test = autoclass('org.jnius.ChildVisibilityTest', include_protected=True, include_private=True)
        test = Test()

        self.assertTrue(hasattr(test, 'methodMultiArgs'))
        self.assertTrue(isinstance(Test.methodMultiArgs, JavaMultipleMethod))

        self.assertTrue(test.methodMultiArgs(True))
        self.assertTrue(test.methodMultiArgs(True, False))
        self.assertTrue(test.methodMultiArgs(True, False, True))

    def test_check_method_vs_field(self):
        """check that HashMap and ArrayList have a size method and not a size field.

        Both the HashMap and ArrayList implementations have a `size()` method
        that is a wrapper for an `int` field also named `size`. The `autoclass`
        function must pick one of these two. If it were to pick the field, this
        would cause an "'int' object is not callable" TypeError when attempting
        to call the size method. This unit test is here to ensure that future
        changes to `autoclass` continue to prefer methods over fields.    
        """

        def assert_is_method(obj, name):
            multiple = "JavaMultipleMethod" in str(type(obj.__class__.__dict__[name]))
            single = "JavaMethod" in str(type(obj.__class__.__dict__[name]))
            self.assertTrue(multiple or single)

        hm = autoclass("java.util.HashMap", include_protected=True, include_private=True)()
        assert_is_method(hm, 'size')

        al = autoclass("java.util.ArrayList", include_protected=True, include_private=True)()
        assert_is_method(al, 'size')

    def test_check_method_vs_property(self):
        """check that "bean" properties don't replace methods.

        The ExecutorService Interface has methods `shutdown()`,  `isShutdown()`,
        `isTerminated()`. The `autoclass` function will create a Python
        `property` if a function name matches a JavaBean name pattern. Those
        properties are important but they should not take priority over a
        method.

        For this Interface it wants to create properties called `shutdown` and
        `terminated` because of `isShutdown` and `isTerminated`. A `shutdown`
        property would conflict with the `shutdown()` method so it should be
        skipped. The `terminated` property is OK though.
        """
        executor = autoclass("java.util.concurrent.Executors")
        pool = executor.newFixedThreadPool(1)

        self.assertTrue(isinstance(pool.__class__.__dict__['shutdown'], JavaMethod))
        self.assertTrue(isinstance(pool.__class__.__dict__['terminated'], property))
        self.assertTrue(isinstance(pool.__class__.__dict__['isShutdown'], JavaMethod))
        self.assertTrue(isinstance(pool.__class__.__dict__['isTerminated'], JavaMethod))



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius import autoclass, java_method, PythonJavaClass

Iterable = autoclass('java.lang.Iterable')
ArrayList = autoclass('java.util.ArrayList')
Runnable = autoclass('java.lang.Runnable')
Thread = autoclass('java.lang.Thread')
Object = autoclass('java.lang.Object')

class SampleIterable(PythonJavaClass):
    __javainterfaces__ = ['java/lang/Iterable']

    @java_method('()Ljava/lang/Iterator;')
    def iterator(self):
        sample = ArrayList()
        sample.add(1)
        sample.add(2)
        return sample.iterator()

class ExportClassTest(unittest.TestCase):
    def test_is_instance(self):
        array_list = ArrayList()
        thread = Thread()
        sample_iterable = SampleIterable()

        self.assertIsInstance(sample_iterable, Iterable)
        self.assertIsInstance(sample_iterable, Object)
        self.assertIsInstance(sample_iterable, SampleIterable)
        self.assertNotIsInstance(sample_iterable, Runnable)
        self.assertNotIsInstance(sample_iterable, Thread)

        self.assertIsInstance(array_list, Iterable)
        self.assertIsInstance(array_list, ArrayList)
        self.assertIsInstance(array_list, Object)

        self.assertNotIsInstance(thread, Iterable)
        self.assertIsInstance(thread, Thread)
        self.assertIsInstance(thread, Runnable)

    def test_subclasses_work_for_arg_matching(self):
        array_list = ArrayList()
        array_list.add(SampleIterable())
        self.assertIsInstance(array_list.get(0), Iterable)
        self.assertIsInstance(array_list.get(0), SampleIterable)


    def assertIsSubclass(self, cls, parent):
        if not issubclass(cls, parent):
            self.fail("%s is not a subclass of %s" %
                      (cls.__name__, parent.__name__))

    def assertNotIsSubclass(self, cls, parent):
        if issubclass(cls, parent):
            self.fail("%s is a subclass of %s" %
                      (cls.__name__, parent.__name__))

    def test_is_subclass(self):
        self.assertIsSubclass(Thread, Runnable)
        self.assertIsSubclass(ArrayList, Iterable)
        self.assertIsSubclass(ArrayList, Object)
        self.assertIsSubclass(SampleIterable, Iterable)
        self.assertNotIsSubclass(Thread, Iterable)
        self.assertNotIsSubclass(ArrayList, Runnable)
        self.assertNotIsSubclass(Runnable, Thread)
        self.assertNotIsSubclass(Iterable, SampleIterable)



from __future__ import absolute_import, unicode_literals
import unittest
from jnius import autoclass, cast, PythonJavaClass, java_method


class _TestImplemIterator(PythonJavaClass):
    __javainterfaces__ = ['java/util/ListIterator']


class _TestImplem(PythonJavaClass):
    __javainterfaces__ = ['java/util/List']

    def __init__(self, *args):
        super(_TestImplem, self).__init__(*args)
        self.data = list(args)

    @java_method('()I')
    def size(self):
        return len(self.data)

    @java_method('(I)Ljava/lang/Object;')
    def get(self, index):
        return self.data[index]

    @java_method('(ILjava/lang/Object;)Ljava/lang/Object;')
    def set(self, index, obj):
        old_object = self.data[index]
        self.data[index] = obj
        return old_object


class TestIntLongConversion(unittest.TestCase):
    def test_reverse(self):
        '''
        String comparison because values are the same for INT and LONG,
        but only __str__ shows the real difference.
        '''

        Collections = autoclass('java.util.Collections')
        List = autoclass('java.util.List')
        pylist = list(range(10))
        a = _TestImplem(*pylist)
        self.assertEqual(a.data, pylist)
        self.assertEqual(str(a.data), '[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]')

        # reverse the array, be sure it's converted back to INT!
        Collections.reverse(a)

        # conversion to/from Java objects hides INT/LONG conv on Py2
        # which is wrong to switch between because even Java
        # recognizes INT and LONG types separately (Py3 doesn't)
        self.assertEqual(a.data, list(reversed(pylist)))
        self.assertNotIn('L', str(a.data))
        self.assertEqual(str(a.data), '[9, 8, 7, 6, 5, 4, 3, 2, 1, 0]')


if __name__ == '__main__':
    unittest.main()



'''
Enum in Java returns itself when trying to get a value, e.g.:

    SimpleEnum.GOOD

is of class instance SimpleEnum.

`javap -s SimpleEnum.class`::

    public final class org.jnius.SimpleEnum ... {
        public static final org.jnius.SimpleEnum GOOD;
            descriptor: Lorg/jnius/SimpleEnum;  <-- this
            ...
'''

from __future__ import absolute_import

import unittest
from jnius.reflect import autoclass


class TestSimpleEnum(unittest.TestCase):
    '''
    Test simple enum from java-src/org/jnius/SimpleEnum.java file.
    '''

    def test_enum(self):
        '''
        Make sure Enum returns something.
        '''
        SimpleEnum = autoclass('org.jnius.SimpleEnum')
        self.assertTrue(SimpleEnum)

    def test_value(self):
        '''
        Test whether the enum values return proper types and strings.
        '''
        SimpleEnum = autoclass('org.jnius.SimpleEnum')

        values = [SimpleEnum.GOOD, SimpleEnum.BAD, SimpleEnum.UGLY]
        for val in values:
            self.assertTrue(val)
            self.assertIsInstance(val, SimpleEnum)
            self.assertEqual(
                type(val),
                type(SimpleEnum.valueOf(val.toString()))
            )

            # 'GOOD', 'BAD', 'UGLY' strings
            self.assertEqual(
                val.toString(),
                SimpleEnum.valueOf(val.toString()).toString()
            )

    def test_value_nested(self):
        '''
        Test that we cover the circular implementation of java.lang.Enum
        that on value returns the parent i.e. the Enum class instead of
        e.g. some Exception or segfault which means we can do Enum.X.X.X...

        Currently does not check anything, ref pyjnius#32:
        https://github.com/kivy/pyjnius/issues/32.
        '''
        _ = autoclass('org.jnius.SimpleEnum')
        # self.assertTrue(SimpleEnum.UGLY.UGLY)


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius import autoclass, JavaException


class AssignableFrom(unittest.TestCase):

    def test_assignable(self):
        ArrayList = autoclass('java.util.ArrayList')
        Object = autoclass('java.lang.Object')

        a = ArrayList()
        # addAll accept Collection, Object must failed
        with self.assertRaisesRegex(TypeError, "Invalid instance of 'java/lang/Object' passed for a 'java/util/Collection'"):
            a.addAll(Object())

        # while adding another ArrayList must be ok.
        a.addAll(ArrayList())





def test_methodcalls():
    from jnius import autoclass
    Parent = autoclass('org.jnius.Parent')
    Child = autoclass('org.jnius.Child')
    child = Child.newInstance()
    parent = Parent.newInstance()
    assert parent.doCall(parent) == 0
    assert child.doCall(0) == 1
    assert child.doCall(child) == 0

def test_fields():
    from jnius import autoclass
    Parent = autoclass('org.jnius.Parent')
    Child = autoclass('org.jnius.Child')
    child = Child.newInstance()
    parent = Parent.newInstance()
    assert parent.PARENT_FIELD == 0
    assert child.CHILD_FIELD == 1
    assert child.PARENT_FIELD == 0

def test_staticfields():
    from jnius import autoclass
    Parent = autoclass('org.jnius.Parent')
    Child = autoclass('org.jnius.Child')
    child = Child.newInstance()
    parent = Parent.newInstance()
    assert Parent.STATIC_PARENT_FIELD == 1
    assert Parent.StaticGetParentStaticField() == 1
    assert Child.STATIC_PARENT_FIELD == 1
    assert Child.StaticGetParentStaticField() == 1
    assert parent.STATIC_PARENT_FIELD == 1
    assert child.STATIC_PARENT_FIELD == 1
    assert parent.StaticGetParentStaticField() == 1
    assert child.StaticGetParentStaticField() == 1
    assert parent.getParentStaticField() == 1
    assert child.getParentStaticField() == 1

    # now test setting in parent
    Parent.STATIC_PARENT_FIELD = 5
    assert parent.STATIC_PARENT_FIELD == 5
    assert Child.STATIC_PARENT_FIELD == 5
    assert child.STATIC_PARENT_FIELD == 5
    assert Parent.StaticGetParentStaticField() == 5
    assert Child.StaticGetParentStaticField() == 5
    assert parent.StaticGetParentStaticField() == 5
    assert child.StaticGetParentStaticField() == 5
    assert parent.getParentStaticField() == 5
    assert child.getParentStaticField() == 5
    
    # now test setting in child
    child.STATIC_PARENT_FIELD = 10
    assert Child.STATIC_PARENT_FIELD == 10
    assert child.STATIC_PARENT_FIELD == 10
    assert Parent.STATIC_PARENT_FIELD == 10
    assert parent.STATIC_PARENT_FIELD == 10
    assert Parent.StaticGetParentStaticField() == 10
    assert Child.StaticGetParentStaticField() == 10
    assert parent.StaticGetParentStaticField() == 10
    assert child.StaticGetParentStaticField() == 10
    assert parent.getParentStaticField() == 10
    assert child.getParentStaticField() == 10

def test_newinstance():
    from jnius import autoclass
    Parent = autoclass('org.jnius.Parent')
    Child = autoclass('org.jnius.Child')

    child = Child.newInstance()
    assert isinstance(child, Child)
    assert isinstance(child, Parent)

if __name__ == "__main__":
    import jnius_config
    #jnius_config.add_options('-Xcheck:jni')
    test_staticfields()




from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius import autoclass

class PassByReferenceOrValueTest(unittest.TestCase):

    def _verify(self, numbers, changed):
        for i in range(len(numbers)):
            self.assertEqual(numbers[i], i * i if changed else i)

    def _verify_all(self, numbers, changed):
            for n, c in zip(numbers, changed):
                self._verify(n, c)

    def test_single_param_static(self):
        VariablePassing = autoclass('org.jnius.VariablePassing')

        # passed by reference (default), numbers should change
        numbers = list(range(10))
        VariablePassing.singleParamStatic(numbers)
        self._verify(numbers, True)

        # passed by reference, numbers should change
        numbers = list(range(10))
        VariablePassing.singleParamStatic(numbers, pass_by_reference=True)
        self._verify(numbers, True)

        # passed by value, numbers should not change
        numbers = list(range(10))
        VariablePassing.singleParamStatic(numbers, pass_by_reference=False)
        self._verify(numbers, False)

    def test_single_param(self):
        VariablePassing = autoclass('org.jnius.VariablePassing')
        variablePassing = VariablePassing()

        # passed by reference (default), numbers should change
        numbers = list(range(10))
        variablePassing.singleParam(numbers)
        self._verify(numbers, True)

        # passed by reference, numbers should change
        numbers = list(range(10))
        variablePassing.singleParam(numbers, pass_by_reference=True)
        self._verify(numbers, True)

        # passed by value, numbers should not change
        numbers = list(range(10))
        variablePassing.singleParam(numbers, pass_by_reference=False)
        self._verify(numbers, False)

    def test_multiple_params_static(self):
        VariablePassing = autoclass('org.jnius.VariablePassing')

        # passed by reference (default), all numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        VariablePassing.multipleParamsStatic(*numbers)
        self._verify_all(numbers, [True] * 4)

        # passed by reference, all numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        VariablePassing.multipleParamsStatic(*numbers, pass_by_reference=True)
        self._verify_all(numbers, [True] * 4)

        # passed by value, no numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        VariablePassing.multipleParamsStatic(*numbers, pass_by_reference=False)
        self._verify_all(numbers, [False] * 4)

        # only the first set of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        VariablePassing.multipleParamsStatic(*numbers, pass_by_reference=[True, False])
        self._verify_all(numbers, [True, False, False, False])

        # only the first set of numbers should not change
        numbers = [list(range(10)) for _ in range(4)]
        VariablePassing.multipleParamsStatic(*numbers, pass_by_reference=[False, True])
        self._verify_all(numbers, [False, True, True, True])

        # only the odd sets of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        changed = (True, False, True, False)
        VariablePassing.multipleParamsStatic(*numbers, pass_by_reference=changed)
        self._verify_all(numbers, changed)

        # only the even sets of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        changed = (False, True, False, True)
        VariablePassing.multipleParamsStatic(*numbers, pass_by_reference=changed)
        self._verify_all(numbers, changed)

    def test_multiple_params(self):
        VariablePassing = autoclass('org.jnius.VariablePassing')
        variablePassing = VariablePassing()

        # passed by reference (default), all numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing.multipleParams(*numbers)
        self._verify_all(numbers, [True] * 4)

        # passed by reference, all numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing.multipleParams(*numbers, pass_by_reference=True)
        self._verify_all(numbers, [True] * 4)

        # passed by value, no numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing.multipleParams(*numbers, pass_by_reference=False)
        self._verify_all(numbers, [False] * 4)

        # only the first set of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing.multipleParams(*numbers, pass_by_reference=[True, False])
        self._verify_all(numbers, [True, False, False, False])

        # only the first set of numbers should not change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing.multipleParams(*numbers, pass_by_reference=[False, True])
        self._verify_all(numbers, [False, True, True, True])

        # only the odd sets of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        changed = (True, False, True, False)
        variablePassing.multipleParams(*numbers, pass_by_reference=changed)
        self._verify_all(numbers, changed)

        # only the even sets of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        changed = (False, True, False, True)
        variablePassing.multipleParams(*numbers, pass_by_reference=changed)
        self._verify_all(numbers, changed)

    def test_contructor_single_param(self):
        VariablePassing = autoclass('org.jnius.VariablePassing')

        # passed by reference (default), numbers should change
        numbers = list(range(10))
        variablePassing = VariablePassing(numbers)
        self._verify(numbers, True)

        # passed by reference, numbers should change
        numbers = list(range(10))
        variablePassing = VariablePassing(numbers, pass_by_reference=True)
        self._verify(numbers, True)

        # passed by value, numbers should not change
        numbers = list(range(10))
        variablePassing = VariablePassing(numbers, pass_by_reference=False)
        self._verify(numbers, False)

    def test_contructor_multiple_params(self):
        VariablePassing = autoclass('org.jnius.VariablePassing')

        # passed by reference (default), all numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing = VariablePassing(*numbers)
        self._verify_all(numbers, [True] * 4)

        # passed by reference, all numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing = VariablePassing(*numbers, pass_by_reference=True)
        self._verify_all(numbers, [True] * 4)

        # passed by value, no numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing = VariablePassing(*numbers, pass_by_reference=False)
        self._verify_all(numbers, [False] * 4)

        # only the first set of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing = VariablePassing(*numbers, pass_by_reference=[True, False])
        self._verify_all(numbers, [True, False, False, False])

        # only the first set of numbers should not change
        numbers = [list(range(10)) for _ in range(4)]
        variablePassing = VariablePassing(*numbers, pass_by_reference=[False, True])
        self._verify_all(numbers, [False, True, True, True])

        # only the odd sets of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        changed = (True, False, True, False)
        variablePassing = VariablePassing(*numbers, pass_by_reference=changed)
        self._verify_all(numbers, changed)

        # only the even sets of numbers should change
        numbers = [list(range(10)) for _ in range(4)]
        changed = (False, True, False, True)
        variablePassing = VariablePassing(*numbers, pass_by_reference=changed)
        self._verify_all(numbers, changed)



'''
Test creating an instance of a Java class and fetching its values.
'''

from __future__ import absolute_import
import unittest
from jnius import autoclass, JavaException


class TestConstructor(unittest.TestCase):
    '''
    TestCase for using constructors with PyJNIus.
    '''

    def test_constructor_none(self):
        '''
        Empty constructor, baked value in the .java file.
        '''

        ConstructorTest = autoclass('org.jnius.ConstructorTest')
        inst = ConstructorTest()
        self.assertEqual(inst.ret, 753)
        self.assertTrue(ConstructorTest._class.isInstance(inst))
        self.assertTrue(inst.getClass().isInstance(inst))

    def test_constructor_int(self):
        '''
        Constructor expecting int and setting it as public 'ret' property.
        '''

        ConstructorTest = autoclass('org.jnius.ConstructorTest')
        inst = ConstructorTest(123)
        self.assertEqual(inst.ret, 123)
        self.assertTrue(ConstructorTest._class.isInstance(inst))
        self.assertTrue(inst.getClass().isInstance(inst))

    def test_constructor_string(self):
        '''
        Constructor expecting char, casting it to int and setting it
        as public 'ret' property.
        '''

        ConstructorTest = autoclass('org.jnius.ConstructorTest')
        inst = ConstructorTest('a')
        self.assertEqual(inst.ret, ord('a'))
        self.assertTrue(ConstructorTest._class.isInstance(inst))
        self.assertTrue(inst.getClass().isInstance(inst))

    def test_constructor_multiobj(self):
        '''
        Constructor expecting String.
        '''

        outputStream = autoclass('java.lang.System').out
        ConstructorTest = autoclass('org.jnius.ConstructorTest')
        inst = ConstructorTest(outputStream, signature="(Ljava/io/OutputStream;)V")
        self.assertEqual(inst.ret, 42)
        self.assertTrue(ConstructorTest._class.isInstance(inst))
        self.assertTrue(inst.getClass().isInstance(inst))

    def test_constructor_int_string(self):
        '''
        Constructor expecting int and char, casting char to int and summing it
        as public 'ret' property.
        '''

        ConstructorTest = autoclass('org.jnius.ConstructorTest')
        inst = ConstructorTest(3, 'a')
        self.assertEqual(inst.ret, 3 + ord('a'))
        self.assertTrue(ConstructorTest._class.isInstance(inst))
        self.assertTrue(inst.getClass().isInstance(inst))

    def test_constructor_exception(self):
        '''
        No such constructor found (no such signature),
        should raise JavaException.
        '''

        ConstructorTest = autoclass('org.jnius.ConstructorTest')
        with self.assertRaises(JavaException):
            ConstructorTest('1.', 2.0, False)


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius.reflect import autoclass
from jnius import cast
from jnius.reflect import identify_hierarchy
from jnius import find_javaclass

def identify_hierarchy_dict(cls, level, concrete=True):
    return({ cls.getName() : level for cls,level in identify_hierarchy(cls, level, concrete) })

class ReflectTest(unittest.TestCase):

    def assertContains(self, d, clsName):
        self.assertTrue(clsName in d, clsName + " was not found in " + str(d))

    def test_hierharchy_queue(self):
        d = identify_hierarchy_dict(find_javaclass("java.util.Queue"), 0, False)
        self.assertContains(d, "java.util.Queue")
        # super interfaces
        self.assertContains(d, "java.util.Collection")
        self.assertContains(d, "java.lang.Iterable")
        # all instantiated interfaces are rooted at Object
        self.assertContains(d, "java.lang.Object")
        maxLevel = max(d.values())
        self.assertEqual(d["java.lang.Object"], maxLevel)
        self.assertEqual(d["java.util.Queue"], 0)
        
    def test_hierharchy_arraylist(self):
        d = identify_hierarchy_dict(find_javaclass("java.util.ArrayList"), 0, True)
        self.assertContains(d, "java.util.ArrayList")# concrete
        self.assertContains(d, "java.util.AbstractCollection")# superclass
        self.assertContains(d, "java.util.Collection")# interface
        self.assertContains(d, "java.lang.Iterable")# interface
        self.assertContains(d, "java.lang.Object")# root
        maxLevel = max(d.values())
        self.assertEqual(d["java.lang.Object"], maxLevel)
        self.assertEqual(d["java.util.ArrayList"], 0)

    def test_class(self):
        lstClz = autoclass("java.util.List")
        self.assertTrue("_class" in dir(lstClz))
        self.assertEqual("java.util.List", lstClz._class.getName())
        alstClz = autoclass("java.util.ArrayList")
        self.assertTrue("_class" in dir(alstClz))
        self.assertEqual("java.util.ArrayList", alstClz._class.getName())
        self.assertEqual("java.util.ArrayList", alstClz().getClass().getName())

    def test_stack(self):
        Stack = autoclass('java.util.Stack')
        stack = Stack()
        self.assertIsInstance(stack, Stack)
        stack.push('hello')
        stack.push('world')
        self.assertEqual(stack.pop(), 'world')
        self.assertEqual(stack.pop(), 'hello')
    
    def test_collection(self):
        HashSet = autoclass('java.util.HashSet')
        aset = HashSet()
        aset.add('hello')
        aset.add('world')
        #check that the __len__ dunder is applied to a Collection not a List
        self.assertEqual(2, len(aset))
        #check that the __len__ dunder is applied to it cast as a Collection
        self.assertEqual(2, len(cast("java.util.Collection", aset)))

    def test_list_interface(self):
        ArrayList = autoclass('java.util.ArrayList')
        words = ArrayList()
        words.add('hello')
        words.add('world')
        self.assertIsNotNone(words.stream())
        self.assertIsNotNone(words.iterator())

    def test_super_interface(self):
        LinkedList = autoclass('java.util.LinkedList')
        words = LinkedList()
        words.add('hello')
        words.add('world')
        q = cast('java.util.Queue', words)
        self.assertEqual(2, q.size())
        self.assertEqual(2, len(q))
        self.assertIsNotNone(q.iterator())

    def test_super_object(self):
        LinkedList = autoclass('java.util.LinkedList')
        words = LinkedList()
        words.hashCode()

    def test_super_interface_object(self):
        LinkedList = autoclass('java.util.LinkedList')
        words = LinkedList()
        q = cast('java.util.Queue', words)
        q.hashCode()

    def test_list_iteration(self):
        ArrayList = autoclass('java.util.ArrayList')
        words = ArrayList()
        words.add('hello')
        words.add('world')
        self.assertEqual(['hello', 'world'], [word for word in words])




from __future__ import absolute_import
import unittest
from jnius import autoclass, protocol_map

class ComparableTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(ComparableTest, self).__init__(*args, **kwargs)
    
    def test_compare_integer(self):
        five = autoclass('java.lang.Integer')(5)
        six = autoclass('java.lang.Integer')(6)
        six_two = autoclass('java.lang.Integer')(6)
        self.assertTrue(five < six)
        self.assertTrue(six > five)
        self.assertTrue(six == six_two)
        self.assertTrue(five <= six)
        self.assertTrue(six >= five)
        self.assertTrue(six >= six_two)
        self.assertTrue(six <= six_two)

if __name__ == '__main__':
    unittest.main()


from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

from jnius import autoclass, JavaException


class Interface(unittest.TestCase):

    def test_reflect_interface(self):
        Interface = autoclass('org.jnius.InterfaceWithPublicEnum')
        self.assertTrue(Interface)

    def test_reflect_enum_in_interface(self):
        ATTITUDE = autoclass('org.jnius.InterfaceWithPublicEnum$ATTITUDE')
        self.assertTrue(ATTITUDE)
        self.assertTrue(ATTITUDE.GOOD)
        self.assertTrue(ATTITUDE.BAD)
        self.assertTrue(ATTITUDE.UGLY)



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius import autoclass

class StringArgumentForByteArrayTest(unittest.TestCase):

    def test_string_arg_for_byte_array(self):
        # the ByteBuffer.wrap() accept only byte[].
        ByteBuffer = autoclass('java.nio.ByteBuffer')
        self.assertIsNotNone(ByteBuffer.wrap(b'hello world'))

    def test_string_arg_with_signed_char(self):
        ByteBuffer = autoclass('java.nio.ByteBuffer')
        self.assertIsNotNone(ByteBuffer.wrap(b'\x00\xffHello World\x7f'))

    def test_fill_byte_array(self):
        arr = [0, 0, 0]
        Test = autoclass('org.jnius.BasicsTest')()
        Test.fillByteArray(arr)
        # we don't received signed byte, but unsigned in python.
        self.assertEqual(
            arr,
            [127, 1, 129])

    def test_create_bytearray(self):
        StringBufferInputStream = autoclass('java.io.StringBufferInputStream')
        nis = StringBufferInputStream("Hello world")
        barr = bytearray("\x00" * 5, encoding="utf8")
        self.assertEqual(nis.read(barr, 0, 5), 5)
        self.assertEqual(barr, b"Hello")

    def test_bytearray_ascii(self):
        ByteArrayInputStream = autoclass('java.io.ByteArrayInputStream')
        s = b"".join(bytes(x) for x in range(256))
        nis = ByteArrayInputStream(s)
        barr = bytearray("\x00" * 256, encoding="ascii")
        self.assertEqual(nis.read(barr, 0, 256), 256)
        self.assertEqual(barr[:256], s[:256])

    def test_empty_bytearray(self):
        Test = autoclass('org.jnius.BasicsTest')()
        arr = Test.methodReturnEmptyByteArray()
        self.assertEqual(len(arr), 0)
        with self.assertRaises(IndexError):
            arr[0]
        self.assertEqual(arr, [])
        self.assertEqual(arr[:1], [])
        self.assertEqual(arr.tostring(), b'')



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
# run it, and check with Java VisualVM if we are eating too much memory or not!
if __name__ == '__main__':
    from jnius import autoclass
    Stack = autoclass('java.util.Stack')
    i = 0
    while True:
        i += 1
        stack = Stack()
        stack.push('hello')



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius.reflect import autoclass
from jnius import cast


class MultipleSignatureTest(unittest.TestCase):
    def test_multiple_constructors(self):
        String = autoclass('java.lang.String')
        s = String('hello world')
        self.assertEqual(s.__javaclass__, 'java/lang/String')
        o = cast('java.lang.Object', s)
        self.assertEqual(o.__javaclass__, 'java/lang/Object')

    def test_mmap_toString(self):
        mapClass = autoclass('java.util.HashMap')
        hmap = mapClass()
        hmap.put("a", "1")
        hmap.toString()
        mmap = cast('java.util.Map', hmap)
        mmap.toString()
        mmap.getClass()



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
import sys
from jnius.reflect import autoclass

try:
    long
except NameError:
    # Python 3
    long = int


class MultipleSignature(unittest.TestCase):

    def test_multiple_constructors(self):
        String = autoclass('java.lang.String')
        self.assertIsNotNone(String('Hello World'))
        self.assertIsNotNone(String(list('Hello World')))
        self.assertIsNotNone(String(list('Hello World'), 3, 5))

    def test_multiple_methods(self):
        String = autoclass('java.lang.String')
        s = String('hello')
        if sys.version_info >= (3, 0):
            self.assertEqual(s.getBytes(), [104, 101, 108, 108, 111])
            self.assertEqual(s.getBytes('utf8'), [104, 101, 108, 108, 111])
        self.assertEqual(s.indexOf(ord('e')), 1)
        self.assertEqual(s.indexOf(ord('e'), 2), -1)

    def test_multiple_constructors_varargs(self):
        VariableArgConstructors = autoclass('org.jnius.VariableArgConstructors')
        c1 = VariableArgConstructors(1, 'var2', 42, None, 4)
        self.assertEqual(c1.constructorUsed, 1)
        c2 = VariableArgConstructors(1, 'var2', None, 4)
        self.assertEqual(c2.constructorUsed, 2)

    def test_multiple_methods_no_args(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve(), 'resolved no args')

    def test_multiple_methods_one_arg(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve('arg'), 'resolved one arg')

    def test_multiple_methods_two_args(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve('one', 'two'), 'resolved two args')

    def test_multiple_methods_two_string_and_an_integer(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve('one', 'two', 1), 'resolved two string and an integer')

    def test_multiple_methods_two_string_and_two_integers(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve('one', 'two', 1, 2), 'resolved two string and two integers')

    def test_multiple_methods_varargs(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve(1, 2, 3), 'resolved varargs')

    def test_multiple_methods_varargs_long(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve(long(1), long(2), long(3)), 'resolved varargs')

    def test_multiple_methods_two_args_and_varargs(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve('one', 'two', 1, 2, 3), 'resolved two args and varargs')

    def test_multiple_methods_one_int_one_small_long_and_a_string(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve(
            1, long(1), "one"), "resolved one int, one long and a string")

    def test_multiple_methods_one_int_one_actual_long_and_a_string(self):
        MultipleMethods = autoclass('org.jnius.MultipleMethods')
        self.assertEqual(MultipleMethods.resolve(
            1, 2 ** 63 - 1, "one"), "resolved one int, one long and a string")



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius import autoclass

class OutputArgs(unittest.TestCase):

    def test_string_output_args(self):
        String = autoclass('java.lang.String')
        string = String('word'.encode('utf-8'))
        btarray= [0] * 4
        string.getBytes(0, 4, btarray, 0)
        self.assertEqual(btarray, [119, 111, 114, 100])



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius import JavaClass, MetaJavaClass, JavaMethod
from six import with_metaclass

class HelloWorldTest(unittest.TestCase):

    def test_helloworld(self):

        class HelloWorld(with_metaclass(MetaJavaClass, JavaClass)):
            __javaclass__ = 'org/jnius/HelloWorld'
            hello = JavaMethod('()Ljava/lang/String;')

        a = HelloWorld()
        self.assertEqual(a.hello(), 'world')



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius.reflect import autoclass

class DirTest(unittest.TestCase):

    def test_varargs_signatures(self):
        # >>> from jnius import autoclass
        # >>> cls = autoclass('java.lang.System')
        # >>> cls.out.printf.signatures()
        # [(['java/lang/String', 'java/lang/Object...'], 'java/io/PrintStream'),
        # (['java/util/Locale', 'java/lang/String', 'java/lang/Object...'], 'java/io/PrintStream')]

        cls = autoclass("java.lang.System")
        result = cls.out.printf.signatures()

        assert isinstance(result, list)
        assert all(isinstance(f, tuple) for f in result)

        assert (['java/lang/String', 'java/lang/Object...'], 'java/io/PrintStream') in result
        assert (['java/util/Locale', 'java/lang/String', 'java/lang/Object...'], 'java/io/PrintStream') in result

    def test_array_signatures(self):
        # >>> from jnius import autoclass
        # >>> cls = autoclass('java.util.List')
        # >>> cls.toArray.signatures()
        # [([], 'java/lang/Object[]'),
        # (['java/lang/Object[]'], 'java/lang/Object[]')]

        cls = autoclass("java.util.List")
        result = cls.toArray.signatures()

        assert isinstance(result, list)
        assert all(isinstance(f, tuple) for f in result)

        assert ([], 'java/lang/Object[]') in result
        assert (['java/lang/Object[]'], 'java/lang/Object[]') in result

    def test_signatures(self):
        # >>> from jnius import autoclass
        # >>> cls = autoclass('java.lang.String')
        # >>> cls.valueOf.signatures()
        # [(['boolean'], 'java/lang/String'),
        #  (['char'], 'java/lang/String'),
        #  (['char[]'], 'java/lang/String'),
        #  (['char[]', 'int', 'int'], 'java/lang/String'),
        #  (['double'], 'java/lang/String'),
        #  (['float'], 'java/lang/String'),
        #  (['int'], 'java/lang/String'),
        #  (['java/lang/Object'], 'java/lang/String'),
        #  (['long'], 'java/lang/String')]

        cls = autoclass("java.lang.String")
        result = cls.valueOf.signatures()

        assert isinstance(result, list)
        assert all(isinstance(f, tuple) for f in result)

        assert sorted(result) == sorted([
            (['boolean'], 'java/lang/String'),
            (['char'], 'java/lang/String'),
            (['char[]'], 'java/lang/String'),
            (['char[]', 'int', 'int'], 'java/lang/String'),
            (['double'], 'java/lang/String'),
            (['float'], 'java/lang/String'),
            (['int'], 'java/lang/String'),
            (['java/lang/Object'], 'java/lang/String'),
            (['long'], 'java/lang/String')
        ])



import sys
import pytest
import jnius_config


class TestJniusConfig:
    def setup_method(self):
        """Resets the options global."""
        jnius_config.options = []
        jnius_config.vm_running = False
        jnius_config.classpath = None

    def teardown_method(self):
        self.setup_method()

    @pytest.mark.parametrize(
        "function,args",
        [
            (jnius_config.set_options, ("option1",)),
            (jnius_config.add_options, ("option1",)),
            (jnius_config.set_classpath, (".",)),
            (jnius_config.add_classpath, (".",)),
        ],
    )
    def test_set_options_vm_running(self, function, args):
        """The functions should only raise an error when the vm is running."""
        assert jnius_config.vm_running is False
        function(*args)
        jnius_config.vm_running = True
        with pytest.raises(ValueError) as ex_info:
            function(*args)
        pytest.mark.skipif(
            sys.version_info < (3, 5), reason="Exception args are different on Python 2"
        )
        assert "VM is already running, can't set" in ex_info.value.args[0]

    def test_set_options(self):
        assert jnius_config.vm_running is False
        assert jnius_config.options == []
        jnius_config.set_options("option1", "option2")
        assert jnius_config.options == ["option1", "option2"]
        jnius_config.set_options("option3")
        assert jnius_config.options == ["option3"]

    def test_add_options(self):
        assert jnius_config.vm_running is False
        assert jnius_config.options == []
        jnius_config.add_options("option1", "option2")
        assert jnius_config.options == ["option1", "option2"]
        jnius_config.add_options("option3")
        assert jnius_config.options == ["option1", "option2", "option3"]

    def test_set_classpath(self):
        assert jnius_config.vm_running is False
        assert jnius_config.classpath is None
        jnius_config.set_classpath(".")
        assert jnius_config.classpath == ["."]
        jnius_config.set_classpath(".", "/usr/local/fem/plugins/*")
        assert jnius_config.classpath == [".", "/usr/local/fem/plugins/*"]

    def test_add_classpath(self):
        assert jnius_config.vm_running is False
        assert jnius_config.classpath is None
        jnius_config.add_classpath(".")
        assert jnius_config.classpath == ["."]
        jnius_config.add_classpath("/usr/local/fem/plugins/*")
        assert jnius_config.classpath == [".", "/usr/local/fem/plugins/*"]



'''
Check various function arguments to be properly passed to Java function
as an Object that is not `null` except `None` itself.
'''

from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import unittest
from jnius import autoclass, JavaException


ObjectArgument = autoclass('org.jnius.ObjectArgument')


class ArgumentsTest(unittest.TestCase):
    '''
    Tests for function arguments.
    '''

    def test_argument_none(self):
        '''
        Converts Python None to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject(None), -1)

    def test_argument_emptylist(self):
        '''
        Converts Python list to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject([]), 0)

    def test_argument_emptytuple(self):
        '''
        Converts Python tuple to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject(()), 0)

    def test_argument_list_emptylist(self):
        '''
        Converts Python list to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject([[], ]), 0)

    def test_argument_tuple_emptytuple(self):
        '''
        Converts Python tuple to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject(((), )), 0)

    def test_argument_list_none(self):
        '''
        Converts Python list to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject([None, ]), 0)

    def test_argument_tuple_none(self):
        '''
        Converts Python tuple to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject((None, )), 0)

    def test_argument_emptyunicode(self):
        '''
        Converts Python unicode to java.lang.Object.
        '''
        self.assertEqual(ObjectArgument.checkObject(u''), 0)

    def test_argument_emptybytes(self):
        '''
        Converts Python bytes to Java String.
        '''
        self.assertEqual(ObjectArgument.checkObject(b''), 0)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import sys
import unittest
from jnius.reflect import autoclass


class ImplementationTest(unittest.TestCase):

    def test_println(self):
        # System.out.println implies recursive lookup, and j_self assignation.
        # It was crashing during the implementation :/
        System = autoclass('java.lang.System')
        System.out.println('')

    def test_printf(self):
        System = autoclass('java.lang.System')
        System.out.printf('hi\n')
        System.out.printf('hi %s %s\n', 'jnius', 'other string')

    def test_unicode(self):
        System = autoclass('java.lang.System')
        System.out.printf(u'é')

        Stack = autoclass('java.util.Stack')
        stack = Stack()
        emoji = u'\U0001F602'
        stack.push(emoji)
        popped = stack.pop()
        if sys.version_info < (3, 0):
            popped = popped.decode('utf-8')
        self.assertEqual(emoji, popped)



import unittest

from jnius import autoclass, java_method, PythonJavaClass, cast

from jnius.signatures import *

JObject = autoclass('java/lang/Object')
JString = autoclass('java/lang/String')
JListIterator = autoclass("java.util.ListIterator")

class _TestImplemIterator(PythonJavaClass):
    __javainterfaces__ = [
        'java/util/ListIterator', ]

    def __init__(self, collection, index=0):
        super(_TestImplemIterator, self).__init__()
        self.collection = collection
        self.index = index

    @with_signature(jboolean, [])
    def hasNext(self):
        return self.index < len(self.collection.data) - 1

    @with_signature(JObject, [])
    def next(self):
        obj = self.collection.data[self.index]
        self.index += 1
        return obj

    @with_signature(jboolean, [])
    def hasPrevious(self):
        return self.index >= 0

    @with_signature(JObject, [])
    def previous(self):
        self.index -= 1
        obj = self.collection.data[self.index]
        return obj

    @with_signature(jint, [])
    def previousIndex(self):
        return self.index - 1

    @with_signature(JString, [])
    def toString(self):
        return repr(self)

    @with_signature(JObject, [jint])
    def get(self, index):
        return self.collection.data[index - 1]

    @with_signature(jvoid, [JObject])
    def set(self, obj):
        self.collection.data[self.index - 1] = obj


class _TestImplem(PythonJavaClass):
    __javainterfaces__ = ['java/util/List']

    def __init__(self, *args):
        super(_TestImplem, self).__init__(*args)
        self.data = list(args)

    @with_signature(autoclass("java.util.Iterator"), [])
    def iterator(self):
        it = _TestImplemIterator(self)
        return it

    @with_signature(JString, [])
    def toString(self):
        return repr(self)

    @with_signature(jint, [])
    def size(self):
        return len(self.data)

    @with_signature(JObject, [jint])
    def get(self, index):
        return self.data[index]

    @with_signature(JObject, [jint, JObject])
    def set(self, index, obj):
        old_object = self.data[index]
        self.data[index] = obj
        return old_object

    @with_signature(JArray(JObject), [])
    def toArray(self):
        return self.data

    @with_signature(JListIterator, [])
    def listIterator(self):
        it = _TestImplemIterator(self)
        return it

    # TODO cover this case of listIterator.
    @java_method(signature(JListIterator, [jint]),
                         name='ListIterator')
    def listIteratorI(self, index):
        it = _TestImplemIterator(self, index)
        return it


from jnius.reflect import autoclass

class SignaturesTest(unittest.TestCase):

    def test_construct_stack_from_testimplem(self):
        Stack = autoclass("java.util.Stack")
        pyjlist = _TestImplem(1, 2, 3, 4, 5, 6, 7)
        stack = Stack()
        stack.addAll(pyjlist)
        self.assertEqual(7, pyjlist.size())
        self.assertEqual(stack.size(), pyjlist.size())
        array = pyjlist.toArray()

    def test_return_types(self):

        # Void
        sig = signature(jvoid, [])
        self.assertEqual(sig, "()V")

        # Boolean
        sig = signature(jboolean, [])
        self.assertEqual(sig, "()Z")

        # Byte
        sig = signature(jbyte, [])
        self.assertEqual(sig, "()B")

        # Char
        sig = signature(jchar, [])
        self.assertEqual(sig, "()C")

        # Double
        sig = signature(jdouble, [])
        self.assertEqual(sig, "()D")

        # Float
        sig = signature(jfloat, [])
        self.assertEqual(sig, "()F")

        # Int
        sig = signature(jint, [])
        self.assertEqual(sig, "()I")

        # Long
        sig = signature(jlong, [])
        self.assertEqual(sig, "()J")

        # Short
        sig = signature(jshort, [])
        self.assertEqual(sig, "()S")

        # Object return method
        String = autoclass("java.lang.String")
        sig = signature(String, [])
        self.assertEqual(sig, "()Ljava/lang/String;")

        # Array return
        sig = signature(JArray(jint), [])
        self.assertEqual(sig, "()[I")

    def test_params(self):
        String = autoclass("java.lang.String")

        # Return void, takes objects as parameters
        sig = signature(jvoid, [String, String])
        self.assertEqual(sig, "(Ljava/lang/String;Ljava/lang/String;)V")

        # Multiple array parameter types
        sig = signature(jvoid, [JArray(jint), JArray(jboolean)])
        self.assertEqual(sig, "([I[Z)V")




from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius.reflect import autoclass

class BasicsTest(unittest.TestCase):

    def test_static_methods(self):
        ClassArgument = autoclass('org.jnius.ClassArgument')
        self.assertEqual(ClassArgument.getName(ClassArgument), 'class org.jnius.ClassArgument')



'''
Test creating for java.io.Closeable dunder
'''

from __future__ import absolute_import
import unittest
from jnius import autoclass, JavaException, protocol_map


class TestCloseable(unittest.TestCase):
    '''
    TestCase for using java.io.Closeable dunder
    '''

    def test_stringreader_closeable(self):
        '''
        this checkes that java.lang.AutoCloseable instances gain
        the correct dunder methods
        '''
        swCheck = autoclass("java.io.StringWriter")()
        self.assertTrue("__enter__" in dir(swCheck))
        self.assertTrue("__exit__" in dir(swCheck))

        with autoclass("java.io.StringWriter")() as sw:
            sw.write("Hello")
            
    def test_our_closeable(self):
        '''
        this checkes that closeable dunder methods are actually called
        org.jnius.CloseableClass has java.io.Closeable, to differ from
        java.lang.AutoCloseable (which is what is in interface_map)
        '''
        ourcloseableClz = autoclass("org.jnius.CloseableClass")
        self.assertTrue("__enter__" in dir(ourcloseableClz()))
        self.assertTrue("__exit__" in dir(ourcloseableClz()))

        self.assertTrue(ourcloseableClz.open)
        with ourcloseableClz() as ourcloseable2:
            self.assertTrue(ourcloseableClz.open)
        self.assertFalse(ourcloseableClz.open)

    def test_protocol_map(self):
       self.assertTrue("java.util.List" in protocol_map)



from __future__ import absolute_import
import unittest
from jnius import autoclass


class ArrayListTest(unittest.TestCase):

    def test_other_dunders(self):
        alist = autoclass('java.util.ArrayList')()
        args = [1,2]
        for arg in args:
            alist.add(arg)
        self.assertEqual(len(args), len(alist))
        for idx, arg in enumerate(args):
            self.assertTrue(arg in alist)
        del(alist[1])
        del(alist[0])
        self.assertEqual(0, len(alist))

    def test_output(self):
        alist = autoclass('java.util.ArrayList')()
        args = [0, 1, 5, -1, -5, 0.0, 1.0, 5.0, -1.0, -5.0, True, False]

        for arg in args:
            alist.add(arg)
        for idx, arg in enumerate(args):
            if isinstance(arg, bool):
                self.assertEqual(str(alist[idx]), str(int(arg)))
            else:
                self.assertEqual(str(alist[idx]), str(arg))
        self.assertEqual(len(args), len(alist))


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest
from jnius.reflect import autoclass

class MultipleDimensionsTest(unittest.TestCase):

    def test_multiple_dimensions(self):
        MultipleDims = autoclass('org.jnius.MultipleDimensions')
        matrix = [[1, 2, 3],
                  [4, 5, 6],
                  [7, 8, 9]]
        self.assertEqual(MultipleDims.methodParamsMatrixI(matrix), True)
        self.assertEqual(MultipleDims.methodReturnMatrixI(), matrix)



# -*- coding: utf-8 -*-
# from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import sys
import unittest
from jnius.reflect import autoclass, JavaException


try:
    long
except NameError:
    # Python 3
    long = int


def py2_encode(uni):
    if sys.version_info < (3, 0):
        uni = uni.encode('utf-8')
    return uni


class CharsAndStringsTest(unittest.TestCase):

    def test_char_fields(self):
        Test = autoclass('org.jnius.CharsAndStrings',
                         include_protected=False, include_private=False)
        test = Test()

        self.assertEqual(test.testChar1, py2_encode('a'))
        if sys.version_info.major >= 3:
            self.assertEqual(test.testChar2, 'ä')
            self.assertEqual(test.testChar3, '☺')

        self.assertEqual(Test.testStaticChar1, py2_encode('a'))
        if sys.version_info.major >= 3:
            self.assertEqual(Test.testStaticChar2, 'ä')
            self.assertEqual(Test.testStaticChar3, '☺')

    def test_string_fields(self):
        Test = autoclass('org.jnius.CharsAndStrings',
                         include_protected=False, include_private=False)
        test = Test()

        self.assertEqual(test.testString1, py2_encode("hello world"))
        if sys.version_info.major >= 3:
            self.assertEqual(test.testString2, "umlauts: äöü")
            self.assertEqual(test.testString3, "happy face: ☺")

        self.assertEqual(Test.testStaticString1, py2_encode("hello world"))
        if sys.version_info.major >= 3:
            self.assertEqual(Test.testStaticString2, "umlauts: äöü")
            self.assertEqual(Test.testStaticString3, "happy face: ☺")

    def test_char_methods(self):
        Test = autoclass('org.jnius.CharsAndStrings',
                         include_protected=False, include_private=False)
        test = Test()

        self.assertEqual(test.testChar(1, py2_encode('a')), py2_encode('a'))
        if sys.version_info.major >= 3:
            self.assertEqual(test.testChar(2, 'ä'), 'ä')
            self.assertEqual(test.testChar(3, '☺'), '☺')

        self.assertEqual(Test.testStaticChar(1, py2_encode('a')), py2_encode('a'))
        if sys.version_info.major >= 3:
            self.assertEqual(Test.testStaticChar(2, 'ä'), 'ä')
            self.assertEqual(Test.testStaticChar(3, '☺'), '☺')

    def test_string_methods(self):
        Test = autoclass('org.jnius.CharsAndStrings',
                         include_protected=False, include_private=False)
        test = Test()

        self.assertEqual(test.testString(1, py2_encode("hello world")), py2_encode("hello world"))
        if sys.version_info.major >= 3:
            self.assertEqual(test.testString(2, "umlauts: äöü"), "umlauts: äöü")
            self.assertEqual(test.testString(3, "happy face: ☺"), "happy face: ☺")

        self.assertEqual(Test.testStaticString(1, py2_encode("hello world")), py2_encode("hello world"))
        if sys.version_info.major >= 3:
            self.assertEqual(Test.testStaticString(2, "umlauts: äöü"), "umlauts: äöü")
            self.assertEqual(Test.testStaticString(3, "happy face: ☺"), "happy face: ☺")

    def test_char_array(self):
        Test = autoclass('org.jnius.CharsAndStrings',
                         include_protected=False, include_private=False)
        test = Test()

        charArray1 = ['a', 'b', 'c']
        charArray2 = ['a', 'ä', '☺']

        for c1, c2 in zip(charArray1, test.testCharArray1):
            self.assertEqual(c1, c2)
        for c1, c2 in zip(charArray1, test.testCharArray(1)):
            self.assertEqual(c1, c2)
        if sys.version_info.major >= 3:
            for c1, c2 in zip(charArray2, test.testCharArray2):
                self.assertEqual(c1, c2)
            for c1, c2 in zip(charArray2, test.testCharArray(2)):
                self.assertEqual(c1, c2)

    def test_static_char_array(self):
        Test = autoclass('org.jnius.CharsAndStrings',
                         include_protected=False, include_private=False)

        charArray1 = ['a', 'b', 'c']
        charArray2 = ['a', 'ä', '☺']

        for c1, c2 in zip(charArray1, Test.testStaticCharArray1):
            self.assertEqual(c1, c2)
        for c1, c2 in zip(charArray1, Test.testStaticCharArray(1)):
            self.assertEqual(c1, c2)
        if sys.version_info.major >= 3:
            for c1, c2 in zip(charArray2, Test.testStaticCharArray2):
                self.assertEqual(c1, c2)
            for c1, c2 in zip(charArray2, Test.testStaticCharArray(2)):
                self.assertEqual(c1, c2)


    def test_java_string(self):
        JString = autoclass('java.lang.String')

        testString1 = JString('hello world')
        self.assertTrue(testString1.equals('hello world'))
        if sys.version_info.major >= 3:
            testString2 = JString('umlauts: äöü')
            self.assertTrue(testString2.equals('umlauts: äöü'))
            testString3 = JString('happy face: ☺')
            self.assertTrue(testString3.equals('happy face: ☺'))

    # two methods below are concerned with type-checking of arguments

    def test_pass_intfloat_as_string(self):
        CharsAndStrings = autoclass("org.jnius.CharsAndStrings")
        self.assertIsNone(CharsAndStrings.testStringDefNull)
        with self.assertRaisesRegex(TypeError, "Invalid instance of 'java/lang/Integer' passed for a 'java/lang/String'"):
            # we cannot provide an int to a String
            CharsAndStrings.setString("a", 2)
        with self.assertRaisesRegex(TypeError, "Invalid instance of 'java/lang/Float' passed for a 'java/lang/String'"):
            # we cannot provide an float to a String
            CharsAndStrings.setString("a", 2.2)
        alist = autoclass("java.util.ArrayList")()
        with self.assertRaisesRegex(TypeError, "Invalid instance of 'java/util/ArrayList' passed for a 'java/lang/String'"):
            # we cannot provide a list to a String
            CharsAndStrings.setString("a", alist)
        system = autoclass("java.lang.System")
        with self.assertRaises(JavaException):
            system.setErr("a string")

    def test_pass_string_as_int(self):
        CharsAndStrings = autoclass("org.jnius.CharsAndStrings")
        self.assertEqual(0, CharsAndStrings.testInt)
        with self.assertRaisesRegex(TypeError, "an integer is required"):
            # we cannot provide a String to an int
            CharsAndStrings.setInt("a", "2")



'''
Test calling non-static methods on classes.
'''

from __future__ import absolute_import
import unittest
from jnius import autoclass, JavaException, JavaMethod, JavaMultipleMethod


class TestStatic(unittest.TestCase):

    def test_method(self):
        '''
        Call a non-static JavaMethod on a class,
        should raise JavaException.
        '''

        String = autoclass('java.lang.String')
        self.assertIsInstance(String.replaceAll, JavaMethod)
        self.assertEqual(String('foo').replaceAll('foo', 'bar'), 'bar')
        with self.assertRaises(JavaException):
            String.replaceAll('foo', 'bar')

    def test_multiplemethod(self):
        '''
        Call a non-static JavaMultipleMethod on a class,
        should raise JavaException.
        '''

        String = autoclass('java.lang.String')
        self.assertIsInstance(String.toString, JavaMultipleMethod)
        self.assertEqual(String('baz').toString(), 'baz')
        with self.assertRaises(JavaException):
            String.toString()


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import sys
import unittest
from jnius.reflect import autoclass

try:
    long
except NameError:
    # Python 3
    long = int

def py2_encode(uni):
    if sys.version_info < (3, 0):
        uni = uni.encode('utf-8')
    return uni


class BasicsTest(unittest.TestCase):

    def test_static_methods(self):
        Test = autoclass('org.jnius.BasicsTest')
        self.assertEqual(Test.methodStaticZ(), True)
        self.assertEqual(Test.methodStaticB(), 127)
        self.assertEqual(Test.methodStaticC(), 'k')
        self.assertEqual(Test.methodStaticS(), 32767)
        self.assertEqual(Test.methodStaticI(), 2147483467)
        self.assertEqual(Test.methodStaticJ(), 9223372036854775807)
        self.assertAlmostEqual(Test.methodStaticF(), 1.23456789)
        self.assertEqual(Test.methodStaticD(), 1.23456789)
        self.assertEqual(Test.methodStaticString(), py2_encode(u'hello \U0001F30E!'))

    def test_static_fields(self):
        Test = autoclass('org.jnius.BasicsTest')
        self.assertEqual(Test.fieldStaticZ, True)
        self.assertEqual(Test.fieldStaticB, 127)
        self.assertEqual(Test.fieldStaticC, 'k')
        self.assertEqual(Test.fieldStaticS, 32767)
        self.assertEqual(Test.fieldStaticI, 2147483467)
        self.assertEqual(Test.fieldStaticJ, 9223372036854775807)
        self.assertAlmostEqual(Test.fieldStaticF, 1.23456789)
        self.assertEqual(Test.fieldStaticD, 1.23456789)
        self.assertEqual(Test.fieldStaticString, py2_encode(u'hello \U0001F30E!'))

    def test_instance_methods(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodZ(), True)
        self.assertEqual(test.methodB(), 127)
        self.assertEqual(test.methodC(), 'k')
        self.assertEqual(test.methodS(), 32767)
        self.assertEqual(test.methodI(), 2147483467)
        self.assertEqual(test.methodJ(), 9223372036854775807)
        self.assertAlmostEqual(test.methodF(), 1.23456789)
        self.assertEqual(test.methodD(), 1.23456789)
        self.assertEqual(test.methodString(), py2_encode(u'hello \U0001F30E!'))

    def test_instance_fields(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.fieldZ, True)
        self.assertEqual(test.fieldB, 127)
        self.assertEqual(test.fieldC, 'k')
        self.assertEqual(test.fieldS, 32767)
        self.assertEqual(test.fieldI, 2147483467)
        self.assertEqual(test.fieldJ, 9223372036854775807)
        self.assertAlmostEqual(test.fieldF, 1.23456789)
        self.assertEqual(test.fieldD, 1.23456789)
        self.assertEqual(test.fieldString, py2_encode(u'hello \U0001F30E!'))
        test2 = autoclass('org.jnius.BasicsTest')(10)
        self.assertEqual(test2.fieldB, 10)
        self.assertEqual(test.fieldB, 127)
        self.assertEqual(test2.fieldB, 10)

    def test_instance_getter_naming(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.disabled, True)
        self.assertEqual(test.enabled, False)

    def test_instance_set_fields(self):
        test = autoclass('org.jnius.BasicsTest')()
        test.fieldSetZ = True
        test.fieldSetB = 127
        test.fieldSetC = ord('k')
        test.fieldSetS = 32767
        test.fieldSetI = 2147483467
        test.fieldSetJ = 9223372036854775807
        test.fieldSetF = 1.23456789
        test.fieldSetD = 1.23456789

        self.assertTrue(test.testFieldSetZ())
        self.assertTrue(test.testFieldSetB())
        self.assertTrue(test.testFieldSetC())
        self.assertTrue(test.testFieldSetS())
        self.assertTrue(test.testFieldSetI())
        self.assertTrue(test.testFieldSetJ())
        self.assertTrue(test.testFieldSetF())
        self.assertTrue(test.testFieldSetD())

    def test_instances_methods_array(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodArrayZ(), [True] * 3)
        self.assertEqual(test.methodArrayB()[0], 127)
        if sys.version_info >= (3, 0):
            self.assertEqual(test.methodArrayB(), [127] * 3)
        self.assertEqual(test.methodArrayC(), ['k'] * 3)
        self.assertEqual(test.methodArrayS(), [32767] * 3)
        self.assertEqual(test.methodArrayI(), [2147483467] * 3)
        self.assertEqual(test.methodArrayJ(), [9223372036854775807] * 3)

        ret = test.methodArrayF()
        ref = [1.23456789] * 3
        self.assertAlmostEqual(ret[0], ref[0])
        self.assertAlmostEqual(ret[1], ref[1])
        self.assertAlmostEqual(ret[2], ref[2])

        self.assertEqual(test.methodArrayD(), [1.23456789] * 3)
        self.assertEqual(test.methodArrayString(), [py2_encode(u'hello \U0001F30E!')] * 3)

    def test_instances_methods_params(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodParamsZBCSIJFD(
            True, 127, 'k', 32767, 2147483467, 9223372036854775807, 1.23456789, 1.23456789), True)
        self.assertEqual(test.methodParamsZBCSIJFD(
            True, long(127), 'k', long(32767), long(2147483467), 9223372036854775807, 1.23456789, 1.23456789), True)
        self.assertEqual(test.methodParamsString(py2_encode(u'hello \U0001F30E!')), True)
        self.assertEqual(test.methodParamsArrayI([1, 2, 3]), True)
        self.assertEqual(test.methodParamsArrayString([
            py2_encode(u'hello'), py2_encode(u'\U0001F30E')]), True)

    def test_instances_methods_params_object_list_str(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodParamsObject([
            'hello', 'world']), True)

    def test_instances_methods_params_object_list_int(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodParamsObject([1, 2]), True)

    def test_instances_methods_params_object_list_float(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodParamsObject([3.14, 1.61]), True)

    def test_instances_methods_params_object_list_long(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodParamsObject([1, 2]), True)

    def test_instances_methods_params_array_byte(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodParamsArrayByte([127, 127, 127]), True)
        ret = test.methodArrayB()
        self.assertEqual(test.methodParamsArrayByte(ret), True)

    def test_return_array_as_object_array_of_strings(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodReturnStrings(), [py2_encode(u'Hello'),
                py2_encode(u'\U0001F30E')])

    def test_return_array_as_object_of_integers(self):
        test = autoclass('org.jnius.BasicsTest')()
        self.assertEqual(test.methodReturnIntegers(), [1, 2])



from time import sleep
from jnius import autoclass

print '-- test hardware start!'

Hardware = autoclass('org.renpy.android.Hardware')
print 'DPI is', Hardware.getDPI()

Hardware.accelerometerEnable(True)
for x in xrange(20):
    print Hardware.accelerometerReading()
    sleep(.1)

print '-- test hardware done!'



from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from six.moves import range
import sys

from jnius import autoclass, java_method, PythonJavaClass, cast

print('1: declare a TestImplem that implement Collection')


class _TestImplemIterator(PythonJavaClass):
    __javainterfaces__ = [
        #'java/util/Iterator',
        'java/util/ListIterator', ]

    def __init__(self, collection, index=0):
        self.collection = collection
        self.index = index

    @java_method('()Z')
    def hasNext(self):
        return self.index < len(self.collection.data) - 1

    @java_method('()Ljava/lang/Object;')
    def next(self):
        obj = self.collection.data[self.index]
        self.index += 1
        return obj

    @java_method('()Z')
    def hasPrevious(self):
        return self.index >= 0

    @java_method('()Ljava/lang/Object;')
    def previous(self):
        self.index -= 1
        obj = self.collection.data[self.index]
        return obj

    @java_method('()I')
    def previousIndex(self):
        return self.index - 1

    @java_method('()Ljava/lang/String;')
    def toString(self):
        return repr(self)

    @java_method('(I)Ljava/lang/Object;')
    def get(self, index):
        return self.collection.data[index - 1]

    @java_method('(Ljava/lang/Object;)V')
    def set(self, obj):
        self.collection.data[self.index - 1] = obj


class _TestImplem(PythonJavaClass):
    __javainterfaces__ = ['java/util/List']

    def __init__(self, *args):
        super(_TestImplem, self).__init__(*args)
        self.data = list(args)

    @java_method('()Ljava/util/Iterator;')
    def iterator(self):
        it = _TestImplemIterator(self)
        return it

    @java_method('()Ljava/lang/String;')
    def toString(self):
        return repr(self)

    @java_method('()I')
    def size(self):
        return len(self.data)

    @java_method('(I)Ljava/lang/Object;')
    def get(self, index):
        return self.data[index]

    @java_method('(ILjava/lang/Object;)Ljava/lang/Object;')
    def set(self, index, obj):
        old_object = self.data[index]
        self.data[index] = obj
        return old_object

    @java_method('()[Ljava/lang/Object;')
    def toArray(self):
        return self.data

    @java_method('()Ljava/util/ListIterator;')
    def listIterator(self):
        it = _TestImplemIterator(self)
        return it

    @java_method('(I)Ljava/util/ListIterator;',
                         name='ListIterator')
    def listIteratorI(self, index):
        it = _TestImplemIterator(self, index)
        return it


class _TestBadSignature(PythonJavaClass):
    __javainterfaces__ = ['java/util/List']

    @java_method('(Landroid/bluetooth/BluetoothDevice;IB[])V')
    def bad_signature(self, *args):
        pass


print('2: instantiate the class, with some data')
a = _TestImplem(*list(range(10)))
print(a)
print(dir(a))

print('tries to get a ListIterator')
iterator = a.listIterator()
print('iterator is', iterator)
while iterator.hasNext():
    print('at index', iterator.index, 'value is', iterator.next())

print('3: Do cast to a collection')
a2 = cast('java/util/Collection', a.j_self)
print(a2)

print('4: Try few method on the collection')
Collections = autoclass('java.util.Collections')
#print Collections.enumeration(a)
#print Collections.enumeration(a)
ret = Collections.max(a)

print("reverse")
print(Collections.reverse(a))
print(a.data)

print("before swap")
print(Collections.swap(a, 2, 3))
print("after swap")
print(a.data)

print("rotate")
print(Collections.rotate(a, 5))
print(a.data)

print('Order of data before shuffle()', a.data)
print(Collections.shuffle(a))
print('Order of data after shuffle()', a.data)


# XXX We have issues for methosd with multiple signature
if sys.version_info >= (3, 0):
    print('-> Collections.max(a)')
    print(Collections.max(a2))
#print '-> Collections.shuffle(a)'
#print Collections.shuffle(a2)

# test bad signature
threw = False
try:
    _TestBadSignature()
except Exception:
    threw = True

if not threw:
    raise Exception("Failed to throw for bad signature")



import unittest

from jnius import autoclass, JavaException

class TestLambdas(unittest.TestCase):

    def testCallable(self):
        callFn = lambda: "done"
        executor = autoclass("java.util.concurrent.Executors").newFixedThreadPool(1)
        future = executor.submit(callFn)
        print(type(future))
        self.assertEqual("done", future.get())
        executor.shutdownNow()

    def testComparator(self):
        numbers = autoclass('java.util.ArrayList')()
        Collections = autoclass('java.util.Collections')
        numbers.add(1)
        numbers.add(3)
        revSort = lambda i, j: j - i 
        Collections.sort(numbers, revSort)
        self.assertEqual(numbers[0], 3)
        self.assertEqual(numbers[1], 1)

    def testJavaUtilFunction(self):
        Collectors = autoclass("java.util.stream.Collectors")
        numbers = autoclass('java.util.ArrayList')()
        numbers.add(1)
        numbers.add(2)
        numbers.add(3)

        def squareFn(i):
            return i * i

        
        #J: List<Integer> squares = numbers.stream().map( i -> i*i).collect(Collectors.toList());
        squares = numbers.stream().map(lambda i : i * i).collect(Collectors.toList())

        self.assertEqual(len(squares), len(numbers))
        self.assertEqual(squares[0], 1)
        self.assertEqual(squares[1], 4)
        self.assertEqual(squares[2], 9)

        squares = numbers.stream().map(squareFn).collect(Collectors.toList())

        self.assertEqual(len(squares), len(numbers))
        self.assertEqual(squares[0], 1)
        self.assertEqual(squares[1], 4)
        self.assertEqual(squares[2], 9)
        

if __name__ == '__main__':
    unittest.main()
